@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2007 IBM Corporation and others.
+ * Copyright (c) 2000, 2009 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -23,26 +23,26 @@ public class EqualExpression extends BinaryExpression {
 		super(left,right,operator);
 	}
 	private void checkNullComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse) {
-		
+
 		LocalVariableBinding local = this.left.localVariableBinding();
 		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
-			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, right.nullStatus(flowInfo), this.left);
+			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, this.right.nullStatus(flowInfo), this.left);
 		}
 		local = this.right.localVariableBinding();
 		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
-			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, left.nullStatus(flowInfo), this.right);
+			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, this.left.nullStatus(flowInfo), this.right);
 		}
 	}
 	private void checkVariableComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse, LocalVariableBinding local, int nullStatus, Expression reference) {
 		switch (nullStatus) {
 			case FlowInfo.NULL :
 				if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-					flowContext.recordUsingNullReference(scope, local, reference, 
+					flowContext.recordUsingNullReference(scope, local, reference,
 							FlowContext.CAN_ONLY_NULL_NON_NULL | FlowContext.IN_COMPARISON_NULL, flowInfo);
 					initsWhenTrue.markAsComparedEqualToNull(local); // from thereon it is set
 					initsWhenFalse.markAsComparedEqualToNonNull(local); // from thereon it is set
 				} else {
-					flowContext.recordUsingNullReference(scope, local, reference, 
+					flowContext.recordUsingNullReference(scope, local, reference,
 							FlowContext.CAN_ONLY_NULL_NON_NULL | FlowContext.IN_COMPARISON_NON_NULL, flowInfo);
 					initsWhenTrue.markAsComparedEqualToNonNull(local); // from thereon it is set
 					initsWhenFalse.markAsComparedEqualToNull(local); // from thereon it is set
@@ -50,73 +50,73 @@ public class EqualExpression extends BinaryExpression {
 				break;
 			case FlowInfo.NON_NULL :
 				if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-					flowContext.recordUsingNullReference(scope, local, reference, 
+					flowContext.recordUsingNullReference(scope, local, reference,
 							FlowContext.CAN_ONLY_NULL | FlowContext.IN_COMPARISON_NON_NULL, flowInfo);
 					initsWhenTrue.markAsComparedEqualToNonNull(local); // from thereon it is set
 				} else {
-					flowContext.recordUsingNullReference(scope, local, reference, 
+					flowContext.recordUsingNullReference(scope, local, reference,
 							FlowContext.CAN_ONLY_NULL | FlowContext.IN_COMPARISON_NULL, flowInfo);
 				}
 				break;
-		}	
+		}
 		// we do not impact enclosing try context because this kind of protection
 		// does not preclude the variable from being null in an enclosing scope
 	}
-	
+
 	public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 		FlowInfo result;
-		if (((bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-			if ((left.constant != Constant.NotAConstant) && (left.constant.typeID() == T_boolean)) {
-				if (left.constant.booleanValue()) { //  true == anything
-					//  this is equivalent to the right argument inits 
-					result = right.analyseCode(currentScope, flowContext, flowInfo);
+		if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
+			if ((this.left.constant != Constant.NotAConstant) && (this.left.constant.typeID() == T_boolean)) {
+				if (this.left.constant.booleanValue()) { //  true == anything
+					//  this is equivalent to the right argument inits
+					result = this.right.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // false == anything
 					//  this is equivalent to the right argument inits negated
-					result = right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = this.right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
-			} 
-			else if ((right.constant != Constant.NotAConstant) && (right.constant.typeID() == T_boolean)) {
-				if (right.constant.booleanValue()) { //  anything == true
-					//  this is equivalent to the left argument inits 
-					result = left.analyseCode(currentScope, flowContext, flowInfo);
+			}
+			else if ((this.right.constant != Constant.NotAConstant) && (this.right.constant.typeID() == T_boolean)) {
+				if (this.right.constant.booleanValue()) { //  anything == true
+					//  this is equivalent to the left argument inits
+					result = this.left.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // anything == false
 					//  this is equivalent to the right argument inits negated
-					result = left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = this.left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
-			} 
+			}
 			else {
-				result = right.analyseCode(
-					currentScope, flowContext, 
-					left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).unconditionalInits();
+				result = this.right.analyseCode(
+					currentScope, flowContext,
+					this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).unconditionalInits();
 			}
 		} else { //NOT_EQUAL :
-			if ((left.constant != Constant.NotAConstant) && (left.constant.typeID() == T_boolean)) {
-				if (!left.constant.booleanValue()) { //  false != anything
-					//  this is equivalent to the right argument inits 
-					result = right.analyseCode(currentScope, flowContext, flowInfo);
+			if ((this.left.constant != Constant.NotAConstant) && (this.left.constant.typeID() == T_boolean)) {
+				if (!this.left.constant.booleanValue()) { //  false != anything
+					//  this is equivalent to the right argument inits
+					result = this.right.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // true != anything
 					//  this is equivalent to the right argument inits negated
-					result = right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = this.right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
 			}
-			else if ((right.constant != Constant.NotAConstant) && (right.constant.typeID() == T_boolean)) {
-				if (!right.constant.booleanValue()) { //  anything != false
-					//  this is equivalent to the right argument inits 
-					result = left.analyseCode(currentScope, flowContext, flowInfo);
+			else if ((this.right.constant != Constant.NotAConstant) && (this.right.constant.typeID() == T_boolean)) {
+				if (!this.right.constant.booleanValue()) { //  anything != false
+					//  this is equivalent to the right argument inits
+					result = this.left.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // anything != true
 					//  this is equivalent to the right argument inits negated
-					result = left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = this.left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
-			} 
+			}
 			else {
-				result = right.analyseCode(
-					currentScope, flowContext, 
-					left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).
+				result = this.right.analyseCode(
+					currentScope, flowContext,
+					this.left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).
 					/* unneeded since we flatten it: asNegatedCondition(). */
 					unconditionalInits();
 			}
 		}
-		if (result instanceof UnconditionalFlowInfo && 
+		if (result instanceof UnconditionalFlowInfo &&
 				(result.tagBits & FlowInfo.UNREACHABLE) == 0) { // the flow info is flat
 			result = FlowInfo.conditional(result.copy(), result.copy());
 			// TODO (maxime) check, reintroduced copy
@@ -124,17 +124,17 @@ public class EqualExpression extends BinaryExpression {
 	  checkNullComparison(currentScope, flowContext, result, result.initsWhenTrue(), result.initsWhenFalse());
 	  return result;
 	}
-	
+
 	public final void computeConstant(TypeBinding leftType, TypeBinding rightType) {
 		if ((this.left.constant != Constant.NotAConstant) && (this.right.constant != Constant.NotAConstant)) {
 			this.constant =
 				Constant.computeConstantOperationEQUAL_EQUAL(
-					left.constant,
+					this.left.constant,
 					leftType.id,
-					right.constant,
+					this.right.constant,
 					rightType.id);
 			if (((this.bits & OperatorMASK) >> OperatorSHIFT) == NOT_EQUAL)
-				constant = BooleanConstant.fromValue(!constant.booleanValue());
+				this.constant = BooleanConstant.fromValue(!this.constant.booleanValue());
 		} else {
 			this.constant = Constant.NotAConstant;
 			// no optimization for null == null
@@ -148,22 +148,22 @@ public class EqualExpression extends BinaryExpression {
 	 * @param valueRequired boolean
 	 */
 	public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
-	
+
 		int pc = codeStream.position;
-		if (constant != Constant.NotAConstant) {
-			if (valueRequired) 
-				codeStream.generateConstant(constant, implicitConversion);
+		if (this.constant != Constant.NotAConstant) {
+			if (valueRequired)
+				codeStream.generateConstant(this.constant, this.implicitConversion);
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
 		}
-		
-		if ((left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
+
+		if ((this.left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
 			generateBooleanEqual(currentScope, codeStream, valueRequired);
 		} else {
 			generateNonBooleanEqual(currentScope, codeStream, valueRequired);
 		}
 		if (valueRequired) {
-			codeStream.generateImplicitConversion(implicitConversion);
+			codeStream.generateImplicitConversion(this.implicitConversion);
 		}
 		codeStream.recordPositionsFrom(pc, this.sourceStart);
 	}
@@ -172,19 +172,19 @@ public class EqualExpression extends BinaryExpression {
 	 *	Optimized operations are: == and !=
 	 */
 	public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
-	
-		if (constant != Constant.NotAConstant) {
+
+		if (this.constant != Constant.NotAConstant) {
 			super.generateOptimizedBoolean(currentScope, codeStream, trueLabel, falseLabel, valueRequired);
 			return;
 		}
-		if (((bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-			if ((left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
+		if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
+			if ((this.left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
 				generateOptimizedBooleanEqual(currentScope, codeStream, trueLabel, falseLabel, valueRequired);
 			} else {
 				generateOptimizedNonBooleanEqual(currentScope, codeStream, trueLabel, falseLabel, valueRequired);
 			}
 		} else {
-			if ((left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
+			if ((this.left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
 				generateOptimizedBooleanEqual(currentScope, codeStream, falseLabel, trueLabel, valueRequired);
 			} else {
 				generateOptimizedNonBooleanEqual(currentScope, codeStream, falseLabel, trueLabel, valueRequired);
@@ -198,18 +198,18 @@ public class EqualExpression extends BinaryExpression {
 	 * Note this code does not optimize conditional constants !!!!
 	 */
 	public void generateBooleanEqual(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
-	
-		// optimized cases: <something equivalent to true> == x, <something equivalent to false> == x, 
-		// optimized cases: <something equivalent to false> != x, <something equivalent to true> != x, 
+
+		// optimized cases: <something equivalent to true> == x, <something equivalent to false> == x,
+		// optimized cases: <something equivalent to false> != x, <something equivalent to true> != x,
 		boolean isEqualOperator = ((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL;
-		Constant cst = left.optimizedBooleanConstant();
+		Constant cst = this.left.optimizedBooleanConstant();
 		if (cst != Constant.NotAConstant) {
-			Constant rightCst = right.optimizedBooleanConstant();
+			Constant rightCst = this.right.optimizedBooleanConstant();
 			if (rightCst != Constant.NotAConstant) {
 				// <something equivalent to true> == <something equivalent to true>, <something equivalent to false> != <something equivalent to true>
 				// <something equivalent to true> == <something equivalent to false>, <something equivalent to false> != <something equivalent to false>
-				left.generateCode(currentScope, codeStream, false);
-				right.generateCode(currentScope, codeStream, false);
+				this.left.generateCode(currentScope, codeStream, false);
+				this.right.generateCode(currentScope, codeStream, false);
 				if (valueRequired) {
 					boolean leftBool = cst.booleanValue();
 					boolean rightBool = rightCst.booleanValue();
@@ -229,17 +229,17 @@ public class EqualExpression extends BinaryExpression {
 				}
 			} else if (cst.booleanValue() == isEqualOperator) {
 				// <something equivalent to true> == x, <something equivalent to false> != x
-				left.generateCode(currentScope, codeStream, false);
-				right.generateCode(currentScope, codeStream, valueRequired);
+				this.left.generateCode(currentScope, codeStream, false);
+				this.right.generateCode(currentScope, codeStream, valueRequired);
 			} else {
 				// <something equivalent to false> == x, <something equivalent to true> != x
 				if (valueRequired) {
 					BranchLabel falseLabel = new BranchLabel(codeStream);
-					left.generateCode(currentScope, codeStream, false);
-					right.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
-					// comparison is TRUE 
+					this.left.generateCode(currentScope, codeStream, false);
+					this.right.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
+					// comparison is TRUE
 					codeStream.iconst_0();
-					if ((bits & IsReturnedValue) != 0){
+					if ((this.bits & IsReturnedValue) != 0){
 						codeStream.generateImplicitConversion(this.implicitConversion);
 						codeStream.generateReturnBytecode(this);
 						// comparison is FALSE
@@ -255,8 +255,8 @@ public class EqualExpression extends BinaryExpression {
 						endLabel.place();
 					}
 				} else {
-					left.generateCode(currentScope, codeStream, false);
-					right.generateCode(currentScope, codeStream, false);
+					this.left.generateCode(currentScope, codeStream, false);
+					this.right.generateCode(currentScope, codeStream, false);
 				}
 //				left.generateCode(currentScope, codeStream, false);
 //				right.generateCode(currentScope, codeStream, valueRequired);
@@ -266,22 +266,22 @@ public class EqualExpression extends BinaryExpression {
 //				}
 			}
 			return;
-		} 
-		cst = right.optimizedBooleanConstant();
+		}
+		cst = this.right.optimizedBooleanConstant();
 		if (cst != Constant.NotAConstant) {
 			if (cst.booleanValue() == isEqualOperator) {
 				// x == <something equivalent to true>, x != <something equivalent to false>
-				left.generateCode(currentScope, codeStream, valueRequired);
-				right.generateCode(currentScope, codeStream, false);
+				this.left.generateCode(currentScope, codeStream, valueRequired);
+				this.right.generateCode(currentScope, codeStream, false);
 			} else {
 				// x == <something equivalent to false>, x != <something equivalent to true>
 				if (valueRequired) {
 					BranchLabel falseLabel = new BranchLabel(codeStream);
-					left.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
-					right.generateCode(currentScope, codeStream, false);
-					// comparison is TRUE 
+					this.left.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
+					this.right.generateCode(currentScope, codeStream, false);
+					// comparison is TRUE
 					codeStream.iconst_0();
-					if ((bits & IsReturnedValue) != 0){
+					if ((this.bits & IsReturnedValue) != 0){
 						codeStream.generateImplicitConversion(this.implicitConversion);
 						codeStream.generateReturnBytecode(this);
 						// comparison is FALSE
@@ -297,8 +297,8 @@ public class EqualExpression extends BinaryExpression {
 						endLabel.place();
 					}
 				} else {
-					left.generateCode(currentScope, codeStream, false);
-					right.generateCode(currentScope, codeStream, false);
+					this.left.generateCode(currentScope, codeStream, false);
+					this.right.generateCode(currentScope, codeStream, false);
 				}
 //				left.generateCode(currentScope, codeStream, valueRequired);
 //				right.generateCode(currentScope, codeStream, false);
@@ -308,18 +308,18 @@ public class EqualExpression extends BinaryExpression {
 //				}
 			}
 			return;
-		} 
+		}
 		// default case
-		left.generateCode(currentScope, codeStream, valueRequired);
-		right.generateCode(currentScope, codeStream, valueRequired);
+		this.left.generateCode(currentScope, codeStream, valueRequired);
+		this.right.generateCode(currentScope, codeStream, valueRequired);
 
 		if (valueRequired) {
 			if (isEqualOperator) {
 				BranchLabel falseLabel;
 				codeStream.if_icmpne(falseLabel = new BranchLabel(codeStream));
-				// comparison is TRUE 
+				// comparison is TRUE
 				codeStream.iconst_1();
-				if ((bits & IsReturnedValue) != 0){
+				if ((this.bits & IsReturnedValue) != 0){
 					codeStream.generateImplicitConversion(this.implicitConversion);
 					codeStream.generateReturnBytecode(this);
 					// comparison is FALSE
@@ -339,28 +339,28 @@ public class EqualExpression extends BinaryExpression {
 			}
 		}
 	}
-	
+
 	/**
 	 * Boolean generation for == with boolean operands
 	 *
 	 * Note this code does not optimize conditional constants !!!!
 	 */
 	public void generateOptimizedBooleanEqual(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
-	
+
 		// optimized cases: true == x, false == x
-		if (left.constant != Constant.NotAConstant) {
-			boolean inline = left.constant.booleanValue();
-			right.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
+		if (this.left.constant != Constant.NotAConstant) {
+			boolean inline = this.left.constant.booleanValue();
+			this.right.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
 			return;
 		} // optimized cases: x == true, x == false
-		if (right.constant != Constant.NotAConstant) {
-			boolean inline = right.constant.booleanValue();
-			left.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
+		if (this.right.constant != Constant.NotAConstant) {
+			boolean inline = this.right.constant.booleanValue();
+			this.left.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
 			return;
 		}
 		// default case
-		left.generateCode(currentScope, codeStream, valueRequired);
-		right.generateCode(currentScope, codeStream, valueRequired);
+		this.left.generateCode(currentScope, codeStream, valueRequired);
+		this.right.generateCode(currentScope, codeStream, valueRequired);
 		if (valueRequired) {
 			if (falseLabel == null) {
 				if (trueLabel != null) {
@@ -377,20 +377,20 @@ public class EqualExpression extends BinaryExpression {
 			}
 		}
 		// reposition the endPC
-		codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);					
+		codeStream.updateLastRecordedEndPC(currentScope, codeStream.position);
 	}
 	/**
 	 * Boolean generation for == with non-boolean operands
 	 *
 	 */
 	public void generateNonBooleanEqual(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
-	
+
 		boolean isEqualOperator = ((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL;
-		if (((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) {
+		if (((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) {
 			Constant cst;
-			if ((cst = left.constant) != Constant.NotAConstant && cst.intValue() == 0) {
+			if ((cst = this.left.constant) != Constant.NotAConstant && cst.intValue() == 0) {
 				// optimized case: 0 == x, 0 != x
-				right.generateCode(currentScope, codeStream, valueRequired);
+				this.right.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					BranchLabel falseLabel = new BranchLabel(codeStream);
 					if (isEqualOperator) {
@@ -398,9 +398,9 @@ public class EqualExpression extends BinaryExpression {
 					} else {
 						codeStream.ifeq(falseLabel);
 					}
-					// comparison is TRUE 
+					// comparison is TRUE
 					codeStream.iconst_1();
-					if ((bits & IsReturnedValue) != 0){
+					if ((this.bits & IsReturnedValue) != 0){
 						codeStream.generateImplicitConversion(this.implicitConversion);
 						codeStream.generateReturnBytecode(this);
 						// comparison is FALSE
@@ -418,9 +418,9 @@ public class EqualExpression extends BinaryExpression {
 				}
 				return;
 			}
-			if ((cst = right.constant) != Constant.NotAConstant && cst.intValue() == 0) {
+			if ((cst = this.right.constant) != Constant.NotAConstant && cst.intValue() == 0) {
 				// optimized case: x == 0, x != 0
-				left.generateCode(currentScope, codeStream, valueRequired);
+				this.left.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					BranchLabel falseLabel = new BranchLabel(codeStream);
 					if (isEqualOperator) {
@@ -428,9 +428,9 @@ public class EqualExpression extends BinaryExpression {
 					} else {
 						codeStream.ifeq(falseLabel);
 					}
-					// comparison is TRUE 
+					// comparison is TRUE
 					codeStream.iconst_1();
-					if ((bits & IsReturnedValue) != 0){
+					if ((this.bits & IsReturnedValue) != 0){
 						codeStream.generateImplicitConversion(this.implicitConversion);
 						codeStream.generateReturnBytecode(this);
 						// comparison is FALSE
@@ -451,8 +451,8 @@ public class EqualExpression extends BinaryExpression {
 		}
 
 		// null cases
-		if (right instanceof NullLiteral) {
-			if (left instanceof NullLiteral) {
+		if (this.right instanceof NullLiteral) {
+			if (this.left instanceof NullLiteral) {
 				// null == null, null != null
 				if (valueRequired) {
 					if (isEqualOperator) {
@@ -463,7 +463,7 @@ public class EqualExpression extends BinaryExpression {
 				}
 			} else {
 				// x == null, x != null
-				left.generateCode(currentScope, codeStream, valueRequired);
+				this.left.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					BranchLabel falseLabel = new BranchLabel(codeStream);
 					if (isEqualOperator) {
@@ -471,9 +471,9 @@ public class EqualExpression extends BinaryExpression {
 					} else {
 						codeStream.ifnull(falseLabel);
 					}
-					// comparison is TRUE 
+					// comparison is TRUE
 					codeStream.iconst_1();
-					if ((bits & IsReturnedValue) != 0){
+					if ((this.bits & IsReturnedValue) != 0){
 						codeStream.generateImplicitConversion(this.implicitConversion);
 						codeStream.generateReturnBytecode(this);
 						// comparison is FALSE
@@ -491,9 +491,9 @@ public class EqualExpression extends BinaryExpression {
 				}
 			}
 			return;
-		} else if (left instanceof NullLiteral) {
+		} else if (this.left instanceof NullLiteral) {
 			// null = x, null != x
-			right.generateCode(currentScope, codeStream, valueRequired);
+			this.right.generateCode(currentScope, codeStream, valueRequired);
 			if (valueRequired) {
 				BranchLabel falseLabel = new BranchLabel(codeStream);
 				if (isEqualOperator) {
@@ -501,9 +501,9 @@ public class EqualExpression extends BinaryExpression {
 				} else {
 					codeStream.ifnull(falseLabel);
 				}
-				// comparison is TRUE 
+				// comparison is TRUE
 				codeStream.iconst_1();
-				if ((bits & IsReturnedValue) != 0){
+				if ((this.bits & IsReturnedValue) != 0){
 					codeStream.generateImplicitConversion(this.implicitConversion);
 					codeStream.generateReturnBytecode(this);
 					// comparison is FALSE
@@ -518,17 +518,17 @@ public class EqualExpression extends BinaryExpression {
 					codeStream.iconst_0();
 					endLabel.place();
 				}
-			}				
+			}
 			return;
 		}
-	
+
 		// default case
-		left.generateCode(currentScope, codeStream, valueRequired);
-		right.generateCode(currentScope, codeStream, valueRequired);
+		this.left.generateCode(currentScope, codeStream, valueRequired);
+		this.right.generateCode(currentScope, codeStream, valueRequired);
 		if (valueRequired) {
 			BranchLabel falseLabel = new BranchLabel(codeStream);
 			if (isEqualOperator) {
-				switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+				switch ((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
 					case T_int :
 						codeStream.if_icmpne(falseLabel);
 						break;
@@ -548,7 +548,7 @@ public class EqualExpression extends BinaryExpression {
 						codeStream.if_acmpne(falseLabel);
 				}
 			} else {
-				switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+				switch ((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
 					case T_int :
 						codeStream.if_icmpeq(falseLabel);
 						break;
@@ -566,11 +566,11 @@ public class EqualExpression extends BinaryExpression {
 						break;
 					default :
 						codeStream.if_acmpeq(falseLabel);
-				}				
+				}
 			}
-			// comparison is TRUE 
+			// comparison is TRUE
 			codeStream.iconst_1();
-			if ((bits & IsReturnedValue) != 0){
+			if ((this.bits & IsReturnedValue) != 0){
 				codeStream.generateImplicitConversion(this.implicitConversion);
 				codeStream.generateReturnBytecode(this);
 				// comparison is FALSE
@@ -584,22 +584,22 @@ public class EqualExpression extends BinaryExpression {
 				falseLabel.place();
 				codeStream.iconst_0();
 				endLabel.place();
-			}			
+			}
 		}
 	}
-	
+
 	/**
 	 * Boolean generation for == with non-boolean operands
 	 *
 	 */
 	public void generateOptimizedNonBooleanEqual(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
-	
+
 		int pc = codeStream.position;
 		Constant inline;
-		if ((inline = right.constant) != Constant.NotAConstant) {
+		if ((inline = this.right.constant) != Constant.NotAConstant) {
 			// optimized case: x == 0
-			if ((((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) && (inline.intValue() == 0)) {
-				left.generateCode(currentScope, codeStream, valueRequired);
+			if ((((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) && (inline.intValue() == 0)) {
+				this.left.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					if (falseLabel == null) {
 						if (trueLabel != null) {
@@ -619,11 +619,11 @@ public class EqualExpression extends BinaryExpression {
 				return;
 			}
 		}
-		if ((inline = left.constant) != Constant.NotAConstant) {
+		if ((inline = this.left.constant) != Constant.NotAConstant) {
 			// optimized case: 0 == x
-			if ((((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int)
+			if ((((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int)
 				&& (inline.intValue() == 0)) {
-				right.generateCode(currentScope, codeStream, valueRequired);
+				this.right.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					if (falseLabel == null) {
 						if (trueLabel != null) {
@@ -645,8 +645,8 @@ public class EqualExpression extends BinaryExpression {
 		}
 		// null cases
 		// optimized case: x == null
-		if (right instanceof NullLiteral) {
-			if (left instanceof NullLiteral) {
+		if (this.right instanceof NullLiteral) {
+			if (this.left instanceof NullLiteral) {
 				// null == null
 				if (valueRequired) {
 					if (falseLabel == null) {
@@ -657,7 +657,7 @@ public class EqualExpression extends BinaryExpression {
 					}
 				}
 			} else {
-				left.generateCode(currentScope, codeStream, valueRequired);
+				this.left.generateCode(currentScope, codeStream, valueRequired);
 				if (valueRequired) {
 					if (falseLabel == null) {
 						if (trueLabel != null) {
@@ -676,8 +676,8 @@ public class EqualExpression extends BinaryExpression {
 			}
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
-		} else if (left instanceof NullLiteral) { // optimized case: null == x
-			right.generateCode(currentScope, codeStream, valueRequired);
+		} else if (this.left instanceof NullLiteral) { // optimized case: null == x
+			this.right.generateCode(currentScope, codeStream, valueRequired);
 			if (valueRequired) {
 				if (falseLabel == null) {
 					if (trueLabel != null) {
@@ -696,15 +696,15 @@ public class EqualExpression extends BinaryExpression {
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
 		}
-	
+
 		// default case
-		left.generateCode(currentScope, codeStream, valueRequired);
-		right.generateCode(currentScope, codeStream, valueRequired);
+		this.left.generateCode(currentScope, codeStream, valueRequired);
+		this.right.generateCode(currentScope, codeStream, valueRequired);
 		if (valueRequired) {
 			if (falseLabel == null) {
 				if (trueLabel != null) {
 					// implicit falling through the FALSE case
-					switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+					switch ((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
 						case T_int :
 							codeStream.if_icmpeq(trueLabel);
 							break;
@@ -727,7 +727,7 @@ public class EqualExpression extends BinaryExpression {
 			} else {
 				// implicit falling through the TRUE case
 				if (trueLabel == null) {
-					switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+					switch ((this.left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
 						case T_int :
 							codeStream.if_icmpne(falseLabel);
 							break;
@@ -757,20 +757,20 @@ public class EqualExpression extends BinaryExpression {
 		return false;
 	}
 	public TypeBinding resolveType(BlockScope scope) {
-	
+
 			boolean leftIsCast, rightIsCast;
-			if ((leftIsCast = left instanceof CastExpression) == true) left.bits |= DisableUnnecessaryCastCheck; // will check later on
-			TypeBinding originalLeftType = left.resolveType(scope);
-	
-			if ((rightIsCast = right instanceof CastExpression) == true) right.bits |= DisableUnnecessaryCastCheck; // will check later on
-			TypeBinding originalRightType = right.resolveType(scope);
-	
+			if ((leftIsCast = this.left instanceof CastExpression) == true) this.left.bits |= DisableUnnecessaryCastCheck; // will check later on
+			TypeBinding originalLeftType = this.left.resolveType(scope);
+
+			if ((rightIsCast = this.right instanceof CastExpression) == true) this.right.bits |= DisableUnnecessaryCastCheck; // will check later on
+			TypeBinding originalRightType = this.right.resolveType(scope);
+
 		// always return BooleanBinding
 		if (originalLeftType == null || originalRightType == null){
-			constant = Constant.NotAConstant;		
+			this.constant = Constant.NotAConstant;
 			return null;
 		}
-	
+
 		// autoboxing support
 		boolean use15specifics = scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5;
 		TypeBinding leftType = originalLeftType, rightType = originalRightType;
@@ -789,66 +789,77 @@ public class EqualExpression extends BinaryExpression {
 		if (leftType.isBaseType() && rightType.isBaseType()) {
 			int leftTypeID = leftType.id;
 			int rightTypeID = rightType.id;
-	
+
 			// the code is an int
 			// (cast)  left   == (cast)  right --> result
 			//  0000   0000       0000   0000      0000
 			//  <<16   <<12       <<8    <<4       <<0
 			int operatorSignature = OperatorSignatures[EQUAL_EQUAL][ (leftTypeID << 4) + rightTypeID];
-			left.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 16) & 0x0000F), originalLeftType);
-			right.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 8) & 0x0000F), originalRightType);
-			bits |= operatorSignature & 0xF;		
+			this.left.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 16) & 0x0000F), originalLeftType);
+			this.right.computeConversion(scope, TypeBinding.wellKnownType(scope, (operatorSignature >>> 8) & 0x0000F), originalRightType);
+			this.bits |= operatorSignature & 0xF;
 			if ((operatorSignature & 0x0000F) == T_undefined) {
-				constant = Constant.NotAConstant;
+				this.constant = Constant.NotAConstant;
 				scope.problemReporter().invalidOperator(this, leftType, rightType);
 				return null;
 			}
 			// check need for operand cast
 			if (leftIsCast || rightIsCast) {
-				CastExpression.checkNeedForArgumentCasts(scope, EQUAL_EQUAL, operatorSignature, left, leftType.id, leftIsCast, right, rightType.id, rightIsCast);
+				CastExpression.checkNeedForArgumentCasts(scope, EQUAL_EQUAL, operatorSignature, this.left, leftType.id, leftIsCast, this.right, rightType.id, rightIsCast);
 			}
 			computeConstant(leftType, rightType);
+
+			// check whether comparing identical expressions
+			Binding leftDirect = Expression.getDirectBinding(this.left);
+			if (leftDirect != null && leftDirect == Expression.getDirectBinding(this.right)) {
+				scope.problemReporter().comparingIdenticalExpressions(this);
+			}
 			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
-	
-		// Object references 
+
+		// Object references
 		// spec 15.20.3
 		if ((!leftType.isBaseType() || leftType == TypeBinding.NULL) // cannot compare: Object == (int)0
 				&& (!rightType.isBaseType() || rightType == TypeBinding.NULL)
-				&& (this.checkCastTypesCompatibility(scope, leftType, rightType, null) 
-						|| this.checkCastTypesCompatibility(scope, rightType, leftType, null))) {
+				&& (checkCastTypesCompatibility(scope, leftType, rightType, null)
+						|| checkCastTypesCompatibility(scope, rightType, leftType, null))) {
 
 			// (special case for String)
 			if ((rightType.id == T_JavaLangString) && (leftType.id == T_JavaLangString)) {
 				computeConstant(leftType, rightType);
 			} else {
-				constant = Constant.NotAConstant;
+				this.constant = Constant.NotAConstant;
 			}
 			TypeBinding objectType = scope.getJavaLangObject();
-			left.computeConversion(scope, objectType, leftType);
-			right.computeConversion(scope, objectType, rightType);
+			this.left.computeConversion(scope, objectType, leftType);
+			this.right.computeConversion(scope, objectType, rightType);
 			// check need for operand cast
-			boolean unnecessaryLeftCast = (left.bits & UnnecessaryCast) != 0;
-			boolean unnecessaryRightCast = (right.bits & UnnecessaryCast) != 0;
+			boolean unnecessaryLeftCast = (this.left.bits & UnnecessaryCast) != 0;
+			boolean unnecessaryRightCast = (this.right.bits & UnnecessaryCast) != 0;
 			if (unnecessaryLeftCast || unnecessaryRightCast) {
-				TypeBinding alternateLeftType = unnecessaryLeftCast ? ((CastExpression)left).expression.resolvedType : leftType;
-				TypeBinding alternateRightType = unnecessaryRightCast ? ((CastExpression)right).expression.resolvedType : rightType;
-				if (this.checkCastTypesCompatibility(scope, alternateLeftType, alternateRightType, null) 
-						|| this.checkCastTypesCompatibility(scope, alternateRightType, alternateLeftType, null)) {
-					if (unnecessaryLeftCast) scope.problemReporter().unnecessaryCast((CastExpression)left); 
-					if (unnecessaryRightCast) scope.problemReporter().unnecessaryCast((CastExpression)right);
+				TypeBinding alternateLeftType = unnecessaryLeftCast ? ((CastExpression)this.left).expression.resolvedType : leftType;
+				TypeBinding alternateRightType = unnecessaryRightCast ? ((CastExpression)this.right).expression.resolvedType : rightType;
+				if (checkCastTypesCompatibility(scope, alternateLeftType, alternateRightType, null)
+						|| checkCastTypesCompatibility(scope, alternateRightType, alternateLeftType, null)) {
+					if (unnecessaryLeftCast) scope.problemReporter().unnecessaryCast((CastExpression)this.left);
+					if (unnecessaryRightCast) scope.problemReporter().unnecessaryCast((CastExpression)this.right);
 				}
 			}
+			// check whether comparing identical expressions
+			Binding leftDirect = Expression.getDirectBinding(this.left);
+			if (leftDirect != null && leftDirect == Expression.getDirectBinding(this.right)) {
+				scope.problemReporter().comparingIdenticalExpressions(this);
+			}
 			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
-		constant = Constant.NotAConstant;
+		this.constant = Constant.NotAConstant;
 		scope.problemReporter().notCompatibleTypesError(this, leftType, rightType);
 		return null;
 	}
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 		if (visitor.visit(this, scope)) {
-			left.traverse(visitor, scope);
-			right.traverse(visitor, scope);
+			this.left.traverse(visitor, scope);
+			this.right.traverse(visitor, scope);
 		}
 		visitor.endVisit(this, scope);
 	}
