@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2007 IBM Corporation and others.
+ * Copyright (c) 2000, 2009 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
 package org.eclipse.jdt.internal.compiler.lookup;
 
 import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
+import org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.eclipse.jdt.internal.compiler.util.HashtableOfObject;
 import org.eclipse.jdt.internal.compiler.util.SimpleSet;
 
@@ -20,8 +21,22 @@ MethodVerifier15(LookupEnvironment environment) {
 	super(environment);
 }
 boolean areMethodsCompatible(MethodBinding one, MethodBinding two) {
-	MethodBinding sub = computeSubstituteMethod(two, one);
-	return sub != null && doesSubstituteMethodOverride(one, sub) && areReturnTypesCompatible(one, sub);
+	// use the original methods to test compatibility, but do not check visibility, etc
+	one = one.original();
+	two = two.original();
+
+	TypeBinding match = one.declaringClass.findSuperTypeOriginatingFrom(two.declaringClass);
+	if (!(match instanceof ReferenceBinding))
+		return false; // method's declaringClass does not inherit from inheritedMethod's
+
+	if (match != two.declaringClass) {
+		MethodBinding[] superMethods = ((ReferenceBinding) match).getMethods(two.selector);
+		for (int i = 0, length = superMethods.length; i < length; i++)
+			if (superMethods[i].original() == two)
+				return isParameterSubsignature(one, superMethods[i]);
+	}
+
+	return isParameterSubsignature(one, two);
 }
 boolean areParametersEqual(MethodBinding one, MethodBinding two) {
 	TypeBinding[] oneArgs = one.parameters;
@@ -43,10 +58,13 @@ boolean areParametersEqual(MethodBinding one, MethodBinding two) {
 			if (!areTypesEqual(oneArgs[i], twoArgs[i])) {
 				if (oneArgs[i].leafComponentType().isRawType()) {
 					if (oneArgs[i].dimensions() == twoArgs[i].dimensions() && oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType())) {
+						// raw mode does not apply if the method defines its own type variables
+						if (one.typeVariables != Binding.NO_TYPE_VARIABLES)
+							return false;
 						// one parameter type is raw, hence all parameters types must be raw or non generic
 						// otherwise we have a mismatch check backwards
 						for (int j = 0; j < i; j++)
-							if (oneArgs[j].leafComponentType().isParameterizedType())
+							if (oneArgs[j].leafComponentType().isParameterizedTypeWithActualArguments())
 								return false;
 						// switch to all raw mode
 						break foundRAW;
@@ -62,7 +80,7 @@ boolean areParametersEqual(MethodBinding one, MethodBinding two) {
 					if (oneArgs[i].dimensions() == twoArgs[i].dimensions() && oneArgs[i].leafComponentType().isEquivalentTo(twoArgs[i].leafComponentType()))
 						continue;
 				return false;
-			} else if (oneArgs[i].leafComponentType().isParameterizedType()) {
+			} else if (oneArgs[i].leafComponentType().isParameterizedTypeWithActualArguments()) {
 				return false; // no remaining parameter can be a Parameterized type (if one has been converted then all RAW types must be converted)
 			}
 		}
@@ -107,22 +125,15 @@ void checkConcreteInheritedMethod(MethodBinding concreteMethod, MethodBinding[]
 
 		// so the parameters are equal and the return type is compatible b/w the currentMethod & the substituted inheritedMethod
 		MethodBinding originalInherited = abstractMethod.original();
-		if (originalInherited.returnType != concreteMethod.returnType) {
-			if (abstractMethod.returnType.leafComponentType().isParameterizedType()) {
-				if (concreteMethod.returnType.leafComponentType().isRawType())
-					problemReporter().unsafeReturnTypeOverride(concreteMethod, originalInherited, this.type);
-			} else if (abstractMethod.hasSubstitutedReturnType() && originalInherited.returnType.leafComponentType().isTypeVariable()) {
-				if (((TypeVariableBinding) originalInherited.returnType.leafComponentType()).declaringElement == originalInherited) { // see 81618 - type variable from inherited method
-					TypeBinding currentReturnType = concreteMethod.returnType.leafComponentType();
-					if (!currentReturnType.isTypeVariable() || ((TypeVariableBinding) currentReturnType).declaringElement != concreteMethod)
-						problemReporter().unsafeReturnTypeOverride(concreteMethod, originalInherited, this.type);
-				}
-			}
-		}
+		if (originalInherited.returnType != concreteMethod.returnType)
+			if (!isAcceptableReturnTypeOverride(concreteMethod, abstractMethod))
+				problemReporter().unsafeReturnTypeOverride(concreteMethod, originalInherited, this.type);
+
 		// check whether bridge method is already defined above for interface methods
-		if (originalInherited.declaringClass.isInterface()
-				&& this.type.superclass.erasure().findSuperTypeWithSameErasure(originalInherited.declaringClass) == null) {
-			this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
+		if (originalInherited.declaringClass.isInterface()) {
+			if ((concreteMethod.declaringClass == this.type.superclass && this.type.superclass.isParameterizedType())
+				|| this.type.superclass.erasure().findSuperTypeOriginatingFrom(originalInherited.declaringClass) == null)
+					this.type.addSyntheticBridgeMethod(originalInherited, concreteMethod.original());
 		}
 	}
 }
@@ -132,32 +143,13 @@ void checkForBridgeMethod(MethodBinding currentMethod, MethodBinding inheritedMe
 
 	// so the parameters are equal and the return type is compatible b/w the currentMethod & the substituted inheritedMethod
 	MethodBinding originalInherited = inheritedMethod.original();
-	if (originalInherited.returnType != currentMethod.returnType) {
-//		if (currentMethod.returnType.needsUncheckedConversion(inheritedMethod.returnType)) {
-//			problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, originalInherited, this.type);
-		if (inheritedMethod.returnType.leafComponentType().isParameterizedType() && currentMethod.returnType.leafComponentType().isRawType()) {
+	if (originalInherited.returnType != currentMethod.returnType)
+		if (!isAcceptableReturnTypeOverride(currentMethod, inheritedMethod))
 			problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, originalInherited, this.type);
-		} else if (inheritedMethod.hasSubstitutedReturnType() && originalInherited.returnType.leafComponentType().isTypeVariable()) {
-			if (((TypeVariableBinding) originalInherited.returnType.leafComponentType()).declaringElement == originalInherited) { // see 81618 - type variable from inherited method
-				TypeBinding currentReturnType = currentMethod.returnType.leafComponentType();
-				if (!currentReturnType.isTypeVariable() || ((TypeVariableBinding) currentReturnType).declaringElement != currentMethod)
-					problemReporter(currentMethod).unsafeReturnTypeOverride(currentMethod, originalInherited, this.type);
-			}
-		}
-	}
 
 	if (this.type.addSyntheticBridgeMethod(originalInherited, currentMethod.original()) != null) {
-		for (int i = 0, l = allInheritedMethods.length; i < l; i++) {
-			MethodBinding otherInheritedMethod = allInheritedMethods[i];
-			MethodBinding otherOriginal = otherInheritedMethod.original();
-			// only check inherited methods that are different & come from separate inheritance paths
-			if (otherOriginal == originalInherited || otherOriginal == otherInheritedMethod) continue;
-			if (inheritedMethod.areParametersEqual(otherInheritedMethod)) continue;
-			// skip it if otherInheritedMethod is defined by a subtype of inheritedMethod's declaringClass
-			if (otherInheritedMethod.declaringClass.erasure() != inheritedMethod.declaringClass.erasure())
-				if (otherInheritedMethod.declaringClass.findSuperTypeWithSameErasure(inheritedMethod.declaringClass) != null)
-					continue;
-			if (detectInheritedNameClash(originalInherited, otherOriginal))
+		for (int i = 0, l = allInheritedMethods == null ? 0 : allInheritedMethods.length; i < l; i++) {
+			if (allInheritedMethods[i] != null && detectInheritedNameClash(originalInherited, allInheritedMethods[i].original()))
 				return;
 		}
 	}
@@ -195,7 +187,7 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 	//		class A implements I<Integer> { public void test(Integer i) {} }
 	//		class B extends A { public void test(Comparable i) {} }
 
-	if (currentMethod.declaringClass.isInterface() || currentMethod.isStatic()) return;
+	if (currentMethod.declaringClass.isInterface() || inheritedMethod.isStatic()) return;
 
 	if (!detectNameClash(currentMethod, inheritedMethod)) { // check up the hierarchy for skipped inherited methods
 		TypeBinding[] currentParams = currentMethod.parameters;
@@ -221,7 +213,7 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 			MethodBinding[] methods = superType.getMethods(currentMethod.selector);
 			for (int m = 0, n = methods.length; m < n; m++) {
 				MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
-				if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
+				if (substitute != null && !isSubstituteParameterSubsignature(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
 					return;
 			}
 			if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
@@ -249,7 +241,7 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 				MethodBinding[] methods = superType.getMethods(currentMethod.selector);
 				for (int m = 0, n = methods.length; m < n; m++){
 					MethodBinding substitute = computeSubstituteMethod(methods[m], currentMethod);
-					if (substitute != null && !doesSubstituteMethodOverride(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
+					if (substitute != null && !isSubstituteParameterSubsignature(currentMethod, substitute) && detectNameClash(currentMethod, substitute))
 						return;
 				}
 				if ((itsInterfaces = superType.superInterfaces()) != Binding.NO_SUPERINTERFACES) {
@@ -270,13 +262,13 @@ void checkForNameClash(MethodBinding currentMethod, MethodBinding inheritedMetho
 void checkInheritedMethods(MethodBinding inheritedMethod, MethodBinding otherInheritedMethod) {
 	// sent from checkMethods() to compare 2 inherited methods that are not 'equal'
 	if (inheritedMethod.declaringClass.erasure() == otherInheritedMethod.declaringClass.erasure()) {
-		if (inheritedMethod.areParameterErasuresEqual(otherInheritedMethod)) {
+		boolean areDuplicates = inheritedMethod.hasSubstitutedParameters() && otherInheritedMethod.hasSubstitutedParameters()
+			? inheritedMethod.areParametersEqual(otherInheritedMethod)
+			: inheritedMethod.areParameterErasuresEqual(otherInheritedMethod);
+		if (areDuplicates) {
 			problemReporter().duplicateInheritedMethods(this.type, inheritedMethod, otherInheritedMethod);
 			return;
 		}
-	} else if (inheritedMethod.declaringClass.findSuperTypeWithSameErasure(otherInheritedMethod.declaringClass) != null) {
-		// skip it if inheritedMethod is defined by a subtype of otherInheritedMethod declaringClass
-		return;
 	}
 
 	// the 2 inherited methods clash because of a parameterized type overrides a raw type
@@ -349,46 +341,21 @@ void checkInheritedMethods(MethodBinding[] methods, int length) {
 
 	super.checkInheritedMethods(methods, length);
 }
-boolean checkInheritedReturnTypes(MethodBinding[] methods, int length) {
-	// assumes length > 1
-	// its possible in 1.5 that A is compatible with B & C, but B is not compatible with C
-	int[] areIncompatible = null;
-	// abstract classes must check every method against each other
-	for (int i = 0, l = this.type.isAbstract() ? length - 2 : 0; i <= l;) {
-		MethodBinding method = methods[i++];
-		nextMethod : for (int j = i; j < length; j++) {
-			if (!areReturnTypesCompatible(method, methods[j])) {
-				if (this.type.isInterface())
-					for (int m = length; --m >= 0;)
-						if (methods[m].declaringClass.id == TypeIds.T_JavaLangObject)
-							continue nextMethod; // do not complain since the super interface already got blamed
-				// check to see if this is just a warning, if so report it & skip to next method
-				if (isUnsafeReturnTypeOverride(method, methods[j])) {
-					problemReporter(method).unsafeReturnTypeOverride(method, methods[j], this.type);
-					continue nextMethod;
-				}
-				if (areIncompatible == null)
-					areIncompatible = new int[length];
-				areIncompatible[i - 1] = -1;
-				areIncompatible[j] = -1;
-			}
-		}
-	}
-	if (areIncompatible == null)
-		return true;
+boolean checkInheritedReturnTypes(MethodBinding method, MethodBinding otherMethod) {
+	if (areReturnTypesCompatible(method, otherMethod)) return true;
 
-	int count = 0;
-	for (int i = 0; i < length; i++)
-		if (areIncompatible[i] == -1) count++;
-	if (count == length) {
-		problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, methods, length);
-		return false;
+	if (!this.type.isInterface())
+		if (method.declaringClass.isClass() || !this.type.implementsInterface(method.declaringClass, false))
+			if (otherMethod.declaringClass.isClass() || !this.type.implementsInterface(otherMethod.declaringClass, false))
+				return true; // do not complain since the superclass already got blamed
+
+	// check to see if this is just a warning, if so report it & skip to next method
+	if (isUnsafeReturnTypeOverride(method, otherMethod)) {
+		if (!method.declaringClass.implementsInterface(otherMethod.declaringClass, false))
+			problemReporter(method).unsafeReturnTypeOverride(method, otherMethod, this.type);
+		return true;
 	}
-	MethodBinding[] methodsToReport = new MethodBinding[count];
-	for (int i = 0, index = 0; i < length; i++)
-		if (areIncompatible[i] == -1)
-			methodsToReport[index++] = methods[i];
-	problemReporter().inheritedMethodsHaveIncompatibleReturnTypes(this.type, methodsToReport, count);
+
 	return false;
 }
 void checkMethods() {
@@ -410,19 +377,27 @@ void checkMethods() {
 		}
 
 		int index = -1;
+		int inheritedLength = inherited.length;
 		MethodBinding[] matchingInherited = new MethodBinding[inherited.length];
-		byte[] foundMatch = new byte[inherited.length];
+		MethodBinding[] foundMatch = new MethodBinding[inherited.length]; // null is no match, otherwise value is matching currentMethod
 		if (current != null) {
 			for (int i = 0, length1 = current.length; i < length1; i++) {
 				MethodBinding currentMethod = current[i];
-				for (int j = 0, length2 = inherited.length; j < length2; j++) {
+				MethodBinding[] nonMatchingInherited = null;
+				for (int j = 0; j < inheritedLength; j++) {
 					MethodBinding inheritedMethod = computeSubstituteMethod(inherited[j], currentMethod);
 					if (inheritedMethod != null) {
-						if (foundMatch[j] == 0 && doesSubstituteMethodOverride(currentMethod, inheritedMethod)) {
+						if (foundMatch[j] == null && isSubstituteParameterSubsignature(currentMethod, inheritedMethod)) {
 							matchingInherited[++index] = inheritedMethod;
-							foundMatch[j] = 1; // cannot null out inherited methods
+							foundMatch[j] = currentMethod;
 						} else {
+							// best place to check each currentMethod against each non-matching inheritedMethod
 							checkForNameClash(currentMethod, inheritedMethod);
+							if (inheritedLength > 1) {
+								if (nonMatchingInherited == null)
+									nonMatchingInherited = new MethodBinding[inheritedLength];
+								nonMatchingInherited[j] = inheritedMethod;
+							}
 						}
 					}
 				}
@@ -430,27 +405,37 @@ void checkMethods() {
 					// see addtional comments in https://bugs.eclipse.org/bugs/show_bug.cgi?id=122881
 					// if (index > 0 && currentMethod.declaringClass.isInterface()) // only check when inherited methods are from interfaces
 					//	checkInheritedReturnTypes(matchingInherited, index + 1);
-					checkAgainstInheritedMethods(currentMethod, matchingInherited, index + 1, inherited); // pass in the length of matching
+					checkAgainstInheritedMethods(currentMethod, matchingInherited, index + 1, nonMatchingInherited); // pass in the length of matching
 					while (index >= 0) matchingInherited[index--] = null; // clear the contents of the matching methods
 				}
 			}
 		}
 
-		for (int i = 0, length = inherited.length; i < length; i++) {
-			if (foundMatch[i] == 1) continue;
-
+		// skip tracks which inherited methods have matched other inherited methods
+		// either because they match the same currentMethod or match each other
+		boolean[] skip = new boolean[inheritedLength];
+		for (int i = 0; i < inheritedLength; i++) {
+			if (skip[i]) continue;
 			MethodBinding inheritedMethod = inherited[i];
-			matchingInherited[++index] = inheritedMethod;
-			for (int j = i + 1; j < length; j++) {
+			MethodBinding matchMethod = foundMatch[i];
+			if (matchMethod == null)
+				matchingInherited[++index] = inheritedMethod;
+			for (int j = i + 1; j < inheritedLength; j++) {
 				MethodBinding otherInheritedMethod = inherited[j];
-				if (foundMatch[j] == 1 || canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
+				if (matchMethod == foundMatch[j] && matchMethod != null)
+					continue; // both inherited methods matched the same currentMethod
+				if (canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
 					continue;
 				otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
 				if (otherInheritedMethod != null) {
-					if (doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod)) {
-						matchingInherited[++index] = otherInheritedMethod;
-						foundMatch[j] = 1; // cannot null out inherited methods
-					} else {
+					if (inheritedMethod.declaringClass != otherInheritedMethod.declaringClass
+						&& isSubstituteParameterSubsignature(inheritedMethod, otherInheritedMethod)) {
+							if (index == -1)
+								matchingInherited[++index] = inheritedMethod;
+							if (foundMatch[j] == null)
+								matchingInherited[++index] = otherInheritedMethod;
+							skip[j] = true;
+					} else if (matchMethod == null && foundMatch[j] == null) {
 						checkInheritedMethods(inheritedMethod, otherInheritedMethod);
 					}
 				}
@@ -459,7 +444,7 @@ void checkMethods() {
 
 			if (index > 0)
 				checkInheritedMethods(matchingInherited, index + 1); // pass in the length of matching
-			else if (mustImplementAbstractMethods && index == 0 && matchingInherited[0].isAbstract())
+			else if (mustImplementAbstractMethods && matchingInherited[0].isAbstract())
 				checkAbstractMethod(matchingInherited[0]);
 			while (index >= 0) matchingInherited[index--] = null; // clear the previous contents of the matching methods
 		}
@@ -484,7 +469,7 @@ void checkTypeVariableMethods(TypeParameter typeParameter) {
 					if (canSkipInheritedMethods(inheritedMethod, otherInheritedMethod))
 						continue;
 					otherInheritedMethod = computeSubstituteMethod(otherInheritedMethod, inheritedMethod);
-					if (otherInheritedMethod != null && doesSubstituteMethodOverride(inheritedMethod, otherInheritedMethod)) {
+					if (otherInheritedMethod != null && isSubstituteParameterSubsignature(inheritedMethod, otherInheritedMethod)) {
 						matchingInherited[++index] = otherInheritedMethod;
 						inherited[j] = null; // do not want to find it again
 					}
@@ -513,25 +498,22 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 		((BinaryTypeBinding) inheritedMethod.declaringClass).resolveTypesFor(inheritedMethod);
 
 	TypeVariableBinding[] inheritedTypeVariables = inheritedMethod.typeVariables;
-	if (inheritedTypeVariables == Binding.NO_TYPE_VARIABLES) return inheritedMethod;
 	int inheritedLength = inheritedTypeVariables.length;
+	if (inheritedLength == 0) return inheritedMethod; // no substitution needed
 	TypeVariableBinding[] typeVariables = currentMethod.typeVariables;
 	int length = typeVariables.length;
-	if (length > 0 && inheritedLength != length) return inheritedMethod; // no match JLS 8.4.2
-	TypeBinding[] arguments = new TypeBinding[inheritedLength];
-	if (inheritedLength <= length) {
-		System.arraycopy(typeVariables, 0, arguments, 0, inheritedLength);
-	} else {
-		System.arraycopy(typeVariables, 0, arguments, 0, length);
-		for (int i = length; i < inheritedLength; i++)
-			arguments[i] = inheritedTypeVariables[i].upperBound();
-	}
-	ParameterizedGenericMethodBinding substitute =
-		this.environment.createParameterizedGenericMethod(inheritedMethod, arguments);
+	if (length == 0)
+		return inheritedMethod.asRawMethod(this.environment);
+	if (length != inheritedLength)
+		return inheritedMethod; // no match JLS 8.4.2
 
 	// interface I { <T> void foo(T t); }
 	// class X implements I { public <T extends I> void foo(T t) {} }
 	// for the above case, we do not want to answer the substitute method since its not a match
+	TypeBinding[] arguments = new TypeBinding[length];
+	System.arraycopy(typeVariables, 0, arguments, 0, length);
+	ParameterizedGenericMethodBinding substitute =
+		this.environment.createParameterizedGenericMethod(inheritedMethod, arguments);
 	for (int i = 0; i < inheritedLength; i++) {
 		TypeVariableBinding inheritedTypeVariable = inheritedTypeVariables[i];
 		TypeBinding argument = arguments[i];
@@ -559,53 +541,46 @@ MethodBinding computeSubstituteMethod(MethodBinding inheritedMethod, MethodBindi
 				return inheritedMethod; // not a match
 			}
 		} else if (inheritedTypeVariable.boundCheck(substitute, argument) != TypeConstants.OK) {
-	    		return inheritedMethod;
+	    	return inheritedMethod;
 		}
 	}
    return substitute;
 }
 boolean detectInheritedNameClash(MethodBinding inherited, MethodBinding otherInherited) {
-	if (!inherited.areParameterErasuresEqual(otherInherited) || inherited.returnType.erasure() != otherInherited.returnType.erasure()) return false;
+	if (!inherited.areParameterErasuresEqual(otherInherited))
+		return false;
+	if (this.environment.globalOptions.sourceLevel < ClassFileConstants.JDK1_7) {
+		// with fix for http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6182950
+		// we now ignore return types when detecting name clashes
+		// FYI for now we will only make this change when compliance is set to 1.7 or higher
+		if (inherited.returnType.erasure() != otherInherited.returnType.erasure())
+			return false;
+	}
+	// skip it if otherInherited is defined by a subtype of inherited's declaringClass
+	if (inherited.declaringClass.erasure() != otherInherited.declaringClass.erasure())
+		if (inherited.declaringClass.findSuperTypeOriginatingFrom(otherInherited.declaringClass) != null)
+			return false;
 
 	problemReporter().inheritedMethodsHaveNameClash(this.type, inherited, otherInherited);
 	return true;
 }
 boolean detectNameClash(MethodBinding current, MethodBinding inherited) {
 	MethodBinding original = inherited.original(); // can be the same as inherited
-	if (!current.areParameterErasuresEqual(original) || current.returnType.erasure() != original.returnType.erasure()) return false;
+	if (!current.areParameterErasuresEqual(original))
+		return false;
+	if (this.environment.globalOptions.sourceLevel < ClassFileConstants.JDK1_7) {
+		// with fix for http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6182950
+		// we now ignore return types when detecting name clashes
+		// FYI for now we will only make this change when compliance is set to 1.7 or higher
+		if (current.returnType.erasure() != original.returnType.erasure())
+			return false;
+	}
 
-	problemReporter(current).methodNameClash(current, 
-			inherited.declaringClass.isRawType() ? inherited : original);
+	problemReporter(current).methodNameClash(current, inherited.declaringClass.isRawType() ? inherited : original);
 	return true;
 }
 public boolean doesMethodOverride(MethodBinding method, MethodBinding inheritedMethod) {
-	MethodBinding substitute = computeSubstituteMethod(inheritedMethod, method);
-	return substitute != null && doesSubstituteMethodOverride(method, substitute);
-}
-boolean doesSubstituteMethodOverride(MethodBinding method, MethodBinding substituteMethod) {
-	if (doTypeVariablesClash(method, substituteMethod)) return false;
-	if (areParametersEqual(method, substituteMethod)) return true;
-	if (method.declaringClass == substituteMethod.declaringClass) return false;
-
-	TypeBinding[] params = method.parameters;
-	TypeBinding[] inheritedParams = substituteMethod.parameters;
-	int length = params.length;
-	if (length != inheritedParams.length)
-		return false;
-
-	// also allow a method such as Number foo(Number) to override <U> T foo(T) where T extends Number
-	if (method.typeVariables != Binding.NO_TYPE_VARIABLES || !substituteMethod.hasSubstitutedParameters())
-		return false;
-
-	for (int i = 0; i < length; i++) {
-		if (inheritedParams[i].kind() == Binding.TYPE_PARAMETER) {
-			if (params[i] != ((TypeVariableBinding) inheritedParams[i]).upperBound())
-				return false;
-		} else if (params[i] != inheritedParams[i]) {
-			return false;			
-		}
-	}
-	return true;
+	return couldMethodOverride(method, inheritedMethod) && areMethodsCompatible(method, inheritedMethod);
 }
 boolean doTypeVariablesClash(MethodBinding one, MethodBinding substituteTwo) {
 	// one has type variables and substituteTwo did not pass bounds check in computeSubstituteMethod()
@@ -680,6 +655,45 @@ SimpleSet findSuperinterfaceCollisions(ReferenceBinding superclass, ReferenceBin
 	}
 	return copy;
 }
+boolean hasGenericParameter(MethodBinding method) {
+	if (method.genericSignature() == null) return false;
+
+	// may be only the return type that is generic, need to check parameters
+	TypeBinding[] params = method.parameters;
+	for (int i = 0, l = params.length; i < l; i++) {
+		TypeBinding param = params[i].leafComponentType();
+		if (param instanceof ReferenceBinding) {
+			int modifiers = ((ReferenceBinding) param).modifiers;
+			if ((modifiers & ExtraCompilerModifiers.AccGenericSignature) != 0)
+				return true;
+		}
+	}
+	return false;
+}
+boolean isAcceptableReturnTypeOverride(MethodBinding currentMethod, MethodBinding inheritedMethod) {
+	// called when currentMethod's return type is compatible with inheritedMethod's return type
+
+	if (inheritedMethod.declaringClass.isRawType())
+		return true; // since the inheritedMethod comes from a raw type, the return type is always acceptable
+
+	MethodBinding originalInherited = inheritedMethod.original();
+	TypeBinding originalInheritedReturnType = originalInherited.returnType.leafComponentType();
+	if (originalInheritedReturnType.isParameterizedTypeWithActualArguments())
+		return !currentMethod.returnType.leafComponentType().isRawType(); // raw types issue a warning if inherited is parameterized
+
+	TypeBinding currentReturnType = currentMethod.returnType.leafComponentType();
+	switch (currentReturnType.kind()) {
+	   	case Binding.TYPE_PARAMETER :
+	   		if (currentReturnType == inheritedMethod.returnType.leafComponentType())
+	   			return true;
+	   		//$FALL-THROUGH$
+		default :
+			if (originalInheritedReturnType.isTypeVariable())
+				if (((TypeVariableBinding) originalInheritedReturnType).declaringElement == originalInherited)
+					return false;
+			return true;
+	}
+}
 // caveat: returns false if a method is implemented that needs a bridge method
 boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBinding existingMethod, ReferenceBinding superType) {
 	if (inheritedMethod.original() != inheritedMethod && existingMethod.declaringClass.isInterface())
@@ -688,9 +702,63 @@ boolean isInterfaceMethodImplemented(MethodBinding inheritedMethod, MethodBindin
 	inheritedMethod = computeSubstituteMethod(inheritedMethod, existingMethod);
 	return inheritedMethod != null
 		&& inheritedMethod.returnType == existingMethod.returnType // keep around to produce bridge methods
-		&& super.isInterfaceMethodImplemented(inheritedMethod, existingMethod, superType);
+		&& doesMethodOverride(existingMethod, inheritedMethod);
+}
+public boolean isMethodSubsignature(MethodBinding method, MethodBinding inheritedMethod) {
+	if (!org.eclipse.jdt.core.compiler.CharOperation.equals(method.selector, inheritedMethod.selector))
+		return false;
+
+	// need to switch back to the original if the method is from a ParameterizedType
+	if (method.declaringClass.isParameterizedType())
+		method = method.original();
+
+	inheritedMethod = inheritedMethod.original();
+	TypeBinding match = method.declaringClass.findSuperTypeOriginatingFrom(inheritedMethod.declaringClass);
+	if ((match instanceof ReferenceBinding) && match != inheritedMethod.declaringClass) {
+		MethodBinding[] superMethods = ((ReferenceBinding) match).getMethods(inheritedMethod.selector);
+		for (int i = 0, length = superMethods.length; i < length; i++)
+			if (superMethods[i].original() == inheritedMethod.original())
+				return isParameterSubsignature(method, superMethods[i]);
+	}
+
+	return isParameterSubsignature(method, inheritedMethod);
+}
+boolean isParameterSubsignature(MethodBinding method, MethodBinding inheritedMethod) {
+	MethodBinding substitute = computeSubstituteMethod(inheritedMethod, method);
+	return substitute != null && isSubstituteParameterSubsignature(method, substitute);
+}
+// if method "overrides" substituteMethod then we can skip over substituteMethod while resolving a message send
+// if it does not then a name clash error is likely
+boolean isSubstituteParameterSubsignature(MethodBinding method, MethodBinding substituteMethod) {
+	if (!areParametersEqual(method, substituteMethod)) {
+		// method can still override substituteMethod in cases like :
+		// <U extends Number> void c(U u) {}
+		// @Override void c(Number n) {}
+		// but method cannot have a "generic-enabled" parameter type
+		if (substituteMethod.hasSubstitutedParameters() && method.areParameterErasuresEqual(substituteMethod))
+			return method.typeVariables == Binding.NO_TYPE_VARIABLES && !hasGenericParameter(method);
+
+		// see https://bugs.eclipse.org/bugs/show_bug.cgi?id=279836
+		if (method.declaringClass.isRawType() && substituteMethod.declaringClass.isRawType())
+			if (method.hasSubstitutedParameters() && substituteMethod.hasSubstitutedParameters())
+				return areMethodsCompatible(method, substituteMethod);
+
+		return false;
+	}
+
+	if (substituteMethod instanceof ParameterizedGenericMethodBinding) {
+		if (method.typeVariables != Binding.NO_TYPE_VARIABLES)
+			return !((ParameterizedGenericMethodBinding) substituteMethod).isRaw;
+		// since substituteMethod has substituted type variables, method cannot have a generic signature AND no variables -> its a name clash if it does
+		return !hasGenericParameter(method);
+	}
+
+	// if method has its own variables, then substituteMethod failed bounds check in computeSubstituteMethod()
+	return method.typeVariables == Binding.NO_TYPE_VARIABLES;
 }
 boolean isUnsafeReturnTypeOverride(MethodBinding currentMethod, MethodBinding inheritedMethod) {
+	// called when currentMethod's return type is NOT compatible with inheritedMethod's return type
+
 	// JLS 3 8.4.5: more are accepted, with an unchecked conversion
 	if (currentMethod.returnType == inheritedMethod.returnType.erasure()) {
 		TypeBinding[] currentParams = currentMethod.parameters;
@@ -701,7 +769,7 @@ boolean isUnsafeReturnTypeOverride(MethodBinding currentMethod, MethodBinding in
 	}
 	if (currentMethod.typeVariables == Binding.NO_TYPE_VARIABLES
 		&& inheritedMethod.original().typeVariables != Binding.NO_TYPE_VARIABLES
-		&& currentMethod.returnType.erasure().findSuperTypeWithSameErasure(inheritedMethod.returnType.erasure()) != null) {
+		&& currentMethod.returnType.erasure().findSuperTypeOriginatingFrom(inheritedMethod.returnType.erasure()) != null) {
 			return true;
 	}
 	return false;
@@ -713,14 +781,14 @@ boolean reportIncompatibleReturnTypeError(MethodBinding currentMethod, MethodBin
 	}
 	return super.reportIncompatibleReturnTypeError(currentMethod, inheritedMethod);
 }
-void verify(SourceTypeBinding someType) {
-	if (someType.isAnnotationType())
-		someType.detectAnnotationCycle();
+void verify() {
+	if (this.type.isAnnotationType())
+		this.type.detectAnnotationCycle();
 
-	super.verify(someType);
+	super.verify();
 
-	for (int i = someType.typeVariables.length; --i >= 0;) {
-		TypeVariableBinding var = someType.typeVariables[i];
+	for (int i = this.type.typeVariables.length; --i >= 0;) {
+		TypeVariableBinding var = this.type.typeVariables[i];
 		// must verify bounds if the variable has more than 1
 		if (var.superInterfaces == Binding.NO_SUPERINTERFACES) continue;
 		if (var.superInterfaces.length == 1 && var.superclass.id == TypeIds.T_JavaLangObject) continue;
@@ -737,7 +805,7 @@ void verify(SourceTypeBinding someType) {
 				: itsInterfaces[j];
 		}
 		computeInheritedMethods(superclass, superInterfaces);
-		checkTypeVariableMethods(someType.scope.referenceContext.typeParameters[i]);
+		checkTypeVariableMethods(this.type.scope.referenceContext.typeParameters[i]);
 	}
 }
 }
